@@ -4,7 +4,9 @@
 #include "log.cpp"
 #endif
 
+#if !defined(GUI) && !defined(SILENT)
 static void RawPrint(char *Msg,MESSAGE_TYPE MessageType);
+#endif
 
 static MESSAGE_TYPE MsgStream=MSG_STDOUT;
 static bool Sound=false;
@@ -19,7 +21,7 @@ void InitConsoleOptions(MESSAGE_TYPE MsgStream,bool Sound)
 #if !defined(GUI) && !defined(SILENT)
 void mprintf(const char *fmt,...)
 {
-  if (MsgStream==MSG_NULL)
+  if (MsgStream==MSG_NULL || MsgStream==MSG_ERRONLY)
     return;
   safebuf char Msg[MaxMsgSize];
   va_list argptr;
@@ -56,6 +58,7 @@ void RawPrint(char *Msg,MESSAGE_TYPE MessageType)
       OutFile.SetHandleType(FILE_HANDLESTD);
       break;
     case MSG_STDERR:
+    case MSG_ERRONLY:
       OutFile.SetHandleType(FILE_HANDLEERR);
       break;
     default:
@@ -118,7 +121,7 @@ void GetPasswordText(char *Str,int MaxLength)
   OemToChar(Str,Str);
   SetConsoleMode(hConIn,ConInMode);
   SetConsoleMode(hConOut,ConOutMode);
-#elif defined(_EMX) || defined(_BEOS)
+#elif defined(_EMX) || defined(_BEOS) || defined(__sparc) || defined(sparc)
   fgets(Str,MaxLength-1,stdin);
 #else
   strncpy(Str,getpass(""),MaxLength-1);
